import json
from pyquery import PyQuery as pq
import six

from django.test import SimpleTestCase
from django.core.exceptions import ValidationError

from ..widgets import JSignatureWidget
from ..settings import JSIGNATURE_HEIGHT


class JSignatureWidgetTest(SimpleTestCase):

    def test_default_media(self):
        widget = JSignatureWidget()
        media = widget.media
        media_js = list(media.render_js())
        self.assertEqual(2, len(media_js))
        media_js_str = "".join(media_js)
        self.assertIn('jSignature.min.js', media_js_str)
        self.assertIn('django_jsignature.js', media_js_str)
        media_css = list(media.render_css())
        self.assertEquals([], media_css)

    def test_init(self):
        w = JSignatureWidget()
        self.assertEquals({}, w.jsignature_attrs)
        given_attrs = {'width': 300, 'height': 100}
        w = JSignatureWidget(jsignature_attrs=given_attrs)
        self.assertEquals(given_attrs, w.jsignature_attrs)

    def test_build_jsignature_id(self):
        w = JSignatureWidget()
        id = w.build_jsignature_id('foo')
        self.assertEqual('jsign_foo', id)

    def test_build_jsignature_config(self):
        w = JSignatureWidget(jsignature_attrs={'width': 400})
        config = w.build_jsignature_config()
        self.assertEqual(400, config.get('width'))
        self.assertEqual(JSIGNATURE_HEIGHT, config.get('height'))

    def test_prep_value_empty_values(self):
        w = JSignatureWidget()
        for val in ['', [], '[]']:
            self.assertEqual('[]', w.prep_value(val))

    def test_prep_value_correct_values_python(self):
        w = JSignatureWidget()
        val = [{"x": [1, 2], "y": [3, 4]}]
        val_prep = w.prep_value(val)
        self.assertIsInstance(val_prep, six.string_types)
        self.assertEquals(val, json.loads(val_prep))

    def test_prep_value_correct_values_json(self):
        w = JSignatureWidget()
        val = [{"x": [1, 2], "y": [3, 4]}]
        val_str = '[{"x":[1,2], "y":[3,4]}]'
        val_prep = w.prep_value(val_str)
        self.assertIsInstance(val_prep, six.string_types)
        self.assertEquals(val, json.loads(val_prep))

    def test_prep_value_incorrect_values(self):
        w = JSignatureWidget()
        val = type('Foo')
        self.assertRaises(ValidationError, w.prep_value, val)

    def test_render(self):
        w = JSignatureWidget()
        output = w.render(name='foo', value=None)
        # Almost useless :/
        self.assertEqual(1, len(pq('.jsign-wrapper', output)))
        self.assertEqual(1, len(pq('[type=hidden]', output)))

    def test_render_reset_button_true(self):
        w = JSignatureWidget(jsignature_attrs={'ResetButton': True})
        output = w.render(name='foo', value=None)
        self.assertEqual(1, len(pq('[type=button]', output)))

    def test_render_reset_button_false(self):
        w = JSignatureWidget(jsignature_attrs={'ResetButton': False})
        output = w.render(name='foo', value=None)
        self.assertEqual(0, len(pq('[type=button]', output)))



import json
import os
import imghdr
from PIL import Image
from django.test import SimpleTestCase

from ..utils import draw_signature

DUMMY_VALUE = [{"x": [205, 210], "y": [59, 63]},
               {"x": [205, 207], "y": [67, 64]}]
DUMMY_STR_VALUE = json.dumps(DUMMY_VALUE)


class UtilsTest(SimpleTestCase):

    def test_inputs_bad_str_value(self):
        self.assertRaises(ValueError, draw_signature, 'foo_bar')

    def test_inputs_bad_type_value(self):
        self.assertRaises(ValueError, draw_signature, object())

    def test_inputs_good_list_value(self):
        draw_signature(DUMMY_VALUE)

    def test_inputs_good_str_value(self):
        draw_signature(DUMMY_STR_VALUE)

    def test_outputs_as_file(self):
        output = draw_signature(DUMMY_VALUE, as_file=True)
        self.assertTrue(os.path.isfile(output))
        self.assertIsNotNone(imghdr.what(output))

    def test_outputs_as_image(self):
        output = draw_signature(DUMMY_VALUE)
        self.assertTrue(issubclass(output.__class__, Image.Image))
        self.assertTrue(all(output.getbbox()))



from django.conf import settings

JSIGNATURE_WIDTH = getattr(
    settings, 'JSIGNATURE_WIDTH', 'ratio')
JSIGNATURE_HEIGHT = getattr(
    settings, 'JSIGNATURE_HEIGHT', 'ratio')
JSIGNATURE_COLOR = getattr(
    settings, 'JSIGNATURE_COLOR', '#000')
JSIGNATURE_BACKGROUND_COLOR = getattr(
    settings, 'JSIGNATURE_BACKGROUND_COLOR', '#FFF')
JSIGNATURE_DECOR_COLOR = getattr(
    settings, 'JSIGNATURE_DECOR_COLOR', '#DDD')
JSIGNATURE_LINE_WIDTH = getattr(
    settings, 'JSIGNATURE_LINE_WIDTH', 0)
JSIGNATURE_UNDO_BUTTON = getattr(
    settings, 'JSIGNATURE_UNDO_BUTTON', False)
JSIGNATURE_RESET_BUTTON = getattr(
    settings, 'JSIGNATURE_RESET_BUTTON', True)

JSIGNATURE_DEFAULT_CONFIG = {
    'width': JSIGNATURE_WIDTH,
    'height': JSIGNATURE_HEIGHT,
    'color': JSIGNATURE_COLOR,
    'background-color': JSIGNATURE_BACKGROUND_COLOR,
    'decor-color': JSIGNATURE_DECOR_COLOR,
    'lineWidth': JSIGNATURE_LINE_WIDTH,
    'UndoButton': JSIGNATURE_UNDO_BUTTON,
    'ResetButton': JSIGNATURE_RESET_BUTTON,
}



"""
    Provides a django model field to store a signature captured
    with jSignature jQuery plugin
"""
import json
import six

from django.db import models
from django.core.exceptions import ValidationError

from .forms import (
    JSignatureField as JSignatureFormField,
    JSIGNATURE_EMPTY_VALUES)


class JSignatureField(six.with_metaclass(models.SubfieldBase, models.Field)):
    """
    A model field handling a signature captured with jSignature
    """
    description = "A signature captured with jSignature"

    def get_internal_type(self):
        return 'TextField'

    def to_python(self, value):
        """
        Validates that the input can be red as a JSON object. Returns a Python
        datetime.date object.
        """
        if value in JSIGNATURE_EMPTY_VALUES:
            return None
        elif isinstance(value, list):
            return value
        try:
            return json.loads(value)
        except ValueError:
            raise ValidationError('Invalid JSON format.')

    def get_prep_value(self, value):
        if value in JSIGNATURE_EMPTY_VALUES:
            return None
        elif isinstance(value, six.string_types):
            return value
        elif isinstance(value, list):
            return json.dumps(value)
        raise ValidationError('Invalid format.')

    def formfield(self, **kwargs):
        defaults = {'form_class': JSignatureFormField}
        defaults.update(kwargs)
        return super(JSignatureField, self).formfield(**defaults)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["jsignature.fields.JSignatureField"])
except ImportError:
    pass



from datetime import date
from django.conf import settings
from django.db.models import loading
from django.test import SimpleTestCase
from django.core.management import call_command

from .models import JSignatureTestModel


class JSignatureFieldsMixinTest(SimpleTestCase):

    def setUp(self):
        self.old_installed_apps = settings.INSTALLED_APPS
        settings.INSTALLED_APPS = list(settings.INSTALLED_APPS)
        settings.INSTALLED_APPS.append('jsignature.tests')
        loading.cache.loaded = False
        call_command('syncdb', verbosity=0)

    def tearDown(self):
        settings.INSTALLED_APPS = self.old_installed_apps

    def test_save_create(self):
        # If an object is created signed, signature date must be set
        signature_value = [{"x": [1, 2], "y": [3, 4]}]
        i = JSignatureTestModel(signature=signature_value)
        i.save()
        i = JSignatureTestModel.objects.get(pk=i.pk)
        self.assertEqual(date.today(), i.signature_date.date())

    def test_save_no_change(self):
        # If signature doesn't change, signature date must not be updated
        signature_value = [{"x": [1, 2], "y": [3, 4]}]
        i = JSignatureTestModel(signature=signature_value)
        i.save()
        i.signature_date = date(2013, 1, 1)
        i.save()
        i = JSignatureTestModel.objects.get(pk=i.pk)
        self.assertEqual(date(2013, 1, 1), i.signature_date.date())

    def test_save_change(self):
        # If signature changes, signature date must be updated too
        signature_value = [{"x": [1, 2], "y": [3, 4]}]
        new_signature_value = [{"x": [5, 6], "y": [7, 8]}]
        i = JSignatureTestModel(signature=signature_value,
                                signature_date=date(2013, 1, 1))
        i.save()
        i.signature_date = date(2013, 1, 1)
        i.signature = new_signature_value
        i.save()
        i = JSignatureTestModel.objects.get(pk=i.pk)
        self.assertEqual(date.today(), i.signature_date.date())

    def test_save_none(self):
        # If sinature is set to None, it must be the same for signature_date
        signature_value = [{"x": [1, 2], "y": [3, 4]}]
        i = JSignatureTestModel(signature=signature_value)
        i.save()
        i.signature = None
        i.save()
        i = JSignatureTestModel.objects.get(pk=i.pk)
        self.assertIsNone(i.signature_date)



"""
    Very inspired by zivezab's django-autograph
    https://github.com/zivezab/django-autograph/blob/master/autograph/utils.py
"""
import json
from itertools import chain
from PIL import Image, ImageDraw, ImageOps

AA = 5  # super sampling gor antialiasing


def draw_signature(data, as_file=False):
    """ Draw signature based on lines stored in json_string.
        `data` can be a json object (list in fact) or a json string
        if `as_file` is True, a temp file is returned instead of Image instance
    """

    if type(data) is str:
        drawing = json.loads(data)
    elif type(data) is list:
        drawing = data
    else:
        raise ValueError

    # Compute box
    width = max(chain(*[d['x'] for d in drawing])) + 10
    height = max(chain(*[d['y'] for d in drawing])) + 10

    # Draw image
    im = Image.new("RGBA", (width*AA, height*AA))
    draw = ImageDraw.Draw(im)
    for line in drawing:
        len_line = len(line['x'])
        points = [(line['x'][i]*AA, line['y'][i]*AA)
                  for i in range(0, len_line)]
        draw.line(points, fill="#000", width=2*AA)
    im = ImageOps.expand(im)
    # Smart crop
    bbox = im.getbbox()
    if bbox:
        im.crop(bbox)

    im.thumbnail((width, height), Image.ANTIALIAS)

    if as_file:
        ret = im._dump(format='PNG')
    else:
        ret = im

    return ret



from .widgets import JSignatureWidgetTest
from .forms import JSignatureFormFieldTest
from .fields import JSignatureFieldTest
from .mixins import JSignatureFieldsMixinTest
from .utils import UtilsTest



from django.test import SimpleTestCase
from django.core.exceptions import ValidationError

from ..widgets import JSignatureWidget
from ..forms import JSignatureField


class JSignatureFormFieldTest(SimpleTestCase):

    def test_widget(self):
        f = JSignatureField()
        self.assertIsInstance(f.widget, JSignatureWidget)

    def test_to_python_empty_values(self):
        f = JSignatureField()
        for val in ['', [], '[]']:
            self.assertIsNone(f.to_python(val))

    def test_to_python_correct_values(self):
        f = JSignatureField()
        val = '[{"x":[1,2], "y":[3,4]}]'
        self.assertEquals([{'x': [1, 2], 'y': [3, 4]}], f.to_python(val))

    def test_to_python_incorrect_values(self):
        f = JSignatureField()
        val = 'foo'
        self.assertRaises(ValidationError, f.to_python, val)



"""
    Provides a django form widget to handle a signature capture field with
    with jSignature jQuery plugin
"""
import json
import six

from django.template.loader import render_to_string
from django.forms.widgets import HiddenInput
from django.core import validators
from django.core.exceptions import ValidationError

from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from jsignature.settings import JSIGNATURE_DEFAULT_CONFIG

JSIGNATURE_EMPTY_VALUES = validators.EMPTY_VALUES + ('[]', )


class JSignatureWidget(HiddenInput):
    """
    A widget handling a signature capture field with with jSignature
    """

    # Actually, this widget has a display so we want it to behave like a
    # normal field, not a hidden one
    is_hidden = False

    class Media:
        js = ('js/jSignature.min.js',
              'js/django_jsignature.js')

    def __init__(self, attrs=None, jsignature_attrs=None):
        super(JSignatureWidget, self).__init__(attrs)
        # Store jSignature js config
        self.jsignature_attrs = jsignature_attrs or {}

    def build_jsignature_config(self):
        """ Build javascript config for jSignature initialization.
            It's a dict with for which default values come from settings
            and can be overriden by jsignature_attrs, given at widget
            instanciation time """
        jsignature_config = JSIGNATURE_DEFAULT_CONFIG.copy()
        jsignature_config.update(self.jsignature_attrs)
        return jsignature_config

    def build_jsignature_id(self, name):
        """ Build HTML id for jsignature container.
            It's important because it's used in javascript code """
        return 'jsign_%s' % name

    def prep_value(self, value):
        """ Prepare value before effectively render widget """
        if value in JSIGNATURE_EMPTY_VALUES:
            return "[]"
        elif isinstance(value, six.string_types):
            return value
        elif isinstance(value, list):
            return json.dumps(value)
        raise ValidationError('Invalid format.')

    def render(self, name, value, attrs=None):
        """ Render widget """
        # Build config
        jsign_id = self.build_jsignature_id(name)
        jsignature_config = self.build_jsignature_config()

        # Prepare value
        value = self.prep_value(value)

        # Build output
        context = {
            'hidden': super(JSignatureWidget, self).render(name, value, attrs),
            'jsign_id': jsign_id,
            'reset_btn_text': _('Reset'),
            'config': jsignature_config,
            'js_config': mark_safe(json.dumps(jsignature_config)),
            'value': mark_safe(value),
        }
        out = render_to_string('jsignature/widget.html', context)

        return mark_safe(out)



"""
    A django mixin providing fields to store a signature captured
    with jSignature jQuery plugin
"""
from datetime import datetime
from django.db import models
from django.utils.translation import ugettext_lazy as _
from .fields import JSignatureField


class JSignatureFieldsMixin(models.Model):
    """ Mixin class providing fields to store a signature with jSignature """
    signature = JSignatureField(
        _('Signature'),
        blank=True,
        null=True)
    signature_date = models.DateTimeField(
        _('Signature date'),
        blank=True,
        null=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):

        is_new = self.pk is None
        original = not is_new and self.__class__.objects.get(pk=self.pk)

        if self.signature:
            if is_new or self.signature != original.signature:
                self.signature_date = datetime.now()
        else:
            self.signature_date = None

        super(JSignatureFieldsMixin, self).save()



import json
import six

from django.test import SimpleTestCase
from django.core.exceptions import ValidationError

from ..fields import JSignatureField
from ..forms import JSignatureField as JSignatureFormField


class JSignatureFieldTest(SimpleTestCase):

    def test_to_python_empty(self):
        f = JSignatureField()
        for val in ['', [], '[]']:
            self.assertIsNone(f.to_python(val))

    def test_to_python_correct_value_python(self):
        f = JSignatureField()
        val = [{"x": [1, 2], "y": [3, 4]}]
        self.assertEquals(val, f.to_python(val))

    def test_to_python_correct_value_json(self):
        f = JSignatureField()
        val = [{"x": [1, 2], "y": [3, 4]}]
        val_str = '[{"x":[1,2], "y":[3,4]}]'
        self.assertEquals(val, f.to_python(val_str))

    def test_to_python_incorrect_value(self):
        f = JSignatureField()
        val = 'foo'
        self.assertRaises(ValidationError, f.to_python, val)

    def test_get_prep_value_empty(self):
        f = JSignatureField()
        for val in ['', [], '[]']:
            self.assertIsNone(f.get_prep_value(val))

    def test_get_prep_value_correct_values_python(self):
        f = JSignatureField()
        val = [{"x": [1, 2], "y": [3, 4]}]
        val_prep = f.get_prep_value(val)
        self.assertIsInstance(val_prep, six.string_types)
        self.assertEquals(val, json.loads(val_prep))

    def test_get_prep_value_correct_values_json(self):
        f = JSignatureField()
        val = [{"x": [1, 2], "y": [3, 4]}]
        val_str = '[{"x":[1,2], "y":[3,4]}]'
        val_prep = f.get_prep_value(val_str)
        self.assertIsInstance(val_prep, six.string_types)
        self.assertEquals(val, json.loads(val_prep))

    def test_get_prep_value_incorrect_values(self):
        f = JSignatureField()
        val = type('Foo')
        self.assertRaises(ValidationError, f.get_prep_value, val)

    def test_formfield(self):
        f = JSignatureField()
        cls = f.formfield().__class__
        self.assertTrue(issubclass(cls, JSignatureFormField))



"""
    Provides a django form field to handle a signature capture field with
    with jSignature jQuery plugin
"""
import json
from django.forms.fields import Field
from django.core import validators
from django.core.exceptions import ValidationError
from .widgets import JSignatureWidget

JSIGNATURE_EMPTY_VALUES = validators.EMPTY_VALUES + ('[]', )


class JSignatureField(Field):
    """
    A field handling a signature capture field with with jSignature
    """
    widget = JSignatureWidget()

    def to_python(self, value):
            """
            Validates that the input can be red as a JSON object.
            Returns a Python list (JSON object unserialized).
            """
            if value in JSIGNATURE_EMPTY_VALUES:
                return None
            try:
                return json.loads(value)
            except ValueError:
                raise ValidationError('Invalid JSON format.')



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

setup(
    name='django-jsignature',
    version='0.8',
    author='Florent Lebreton',
    author_email='florent.lebreton@makina-corpus.com',
    url='https://github.com/fle/django-jsignature',
    download_url='https://github.com/fle/django-jsignature/tarball/0.8',
    description='Use jSignature jQuery plugin in your django projects',
    long_description=open(os.path.join(here, 'README.rst')).read() + '\n\n' +
        open(os.path.join(here, 'CHANGES')).read(),
    license='LPGL, see LICENSE file.',
    install_requires=['Django'],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Topic :: Utilities',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Development Status :: 5 - Production/Stable',
         'Programming Language :: Python',
         'Programming Language :: Python :: 2',
         'Programming Language :: Python :: 2.6',
         'Programming Language :: Python :: 2.7',
         'Programming Language :: Python :: 3',
         'Programming Language :: Python :: 3.2',
         'Programming Language :: Python :: 3.3',
         'Programming Language :: Python :: 3.4'
    ],
)
